# codebook.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

class Chapter:
    """Chapter contains a list of used glyph about a face."""
    def __init__(self):
        self._usedGlyphList = {}
    
    def addUsedGlyphs(self, used_char_list):
        for i in used_char_list:
            self._usedGlyphList[i] = True
    
    def getUsedGlyphs(self):
        return self._usedGlyphList.keys()
    
    def hasGlyph(self, glyphCode):
        return self._usedGlyphList.has_key(glyphCode)

import unicodedata
import random
import copy
import string
from util import FaceId, encode, decode
from gnosis.xml.pickle import XML_Pickler
from pangosweeper import PangoSweeper

class Codebook:
    def __init__(self):
        self._chapterDic = {}
        self._cipherDic = {}
        self._faceDic = {}
        alph = u'abcdefghijklmnopqrstuvwwyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        # string.ascii_letters is not unicode string
        alph_l = list(alph)
        alph_l.sort()
        self._facesublist = alph_l # use to substitute face name generation
        self._pangoSweeper = PangoSweeper()
        self._pangoSweeper.setCodebook(self)

        self._charDic = {}
        f = file('shufflable.conf', 'r')
        d = XML_Pickler().load(f)
        f.close()
        for k in d.keys():
            l_org = list(d[k])
            l_cip = copy.copy(l_org)
            random.shuffle(l_cip)
            for o, c in zip(l_org, l_cip):
                self._charDic[o] = c
    
    def cipherize(self, pangotag_list, content):
        """Sweep content and return cipherized content."""
        self._pangoSweeper.sweep(pangotag_list, content)

        cl = list(content)
        ccl = []
        for i in cl:
            if not self._cipherDic.has_key(i):
                if self._charDic.has_key(i):
                    self._cipherDic[i] = self._charDic[i]
                else:
                    self._cipherDic[i] = i
            ccl.append(self._cipherDic[i])
        return string.join(ccl, '')
    
    def getCipherDic(self):
        return self._cipherDic
    
    def getSubstituteFace(self, original):
        if not self._faceDic.has_key(original):
            random.shuffle(self._facesublist)
            sn = string.join(self._facesublist[0:8], '')
            self._faceDic[original] = sn
            self._faceDic[sn] = sn
        return self._faceDic[original]

    def addUsedFaceGlyphs(self, filename, index, glyph_list):
        id = FaceId(filename, index)
        if not self._chapterDic.has_key(id):
            self._chapterDic[id] = Chapter()
        self._chapterDic[id].addUsedGlyphs(glyph_list)
    
    def getChapter(self, faceid):
        return self._chapterDic[faceid]
    
    def getChapterDic(self):
        dic = {}
        for faceid, chapter in self._chapterDic.iteritems():
            ug = chapter.getUsedGlyphs()
            if len(ug) > 0:
                dic[faceid] = chapter
        return dic

    def getCipheredChar(self, char):
        d = self._cipherDic;
        if not self._cipherDic.has_key(char):
            return char
        return self._cipherDic[char]

class ProxyCodebook:
    def setCodebook(self, codebook):
        self._codebook = codebook

    def addUsedFaceGlyphs(self, face, index, cid_list):
        self._codebook.addUsedFaceGlyphs(decode(face), index, cid_list)

    def getCipheredChar(self, char_int):
        char = unichr(char_int)
        cc = self._codebook.getCipheredChar(char)
        return ord(cc)
    
    def getSubstituteFace(self, original):
        return encode(self._codebook.getSubstituteFace(decode(original)))
