# -*- coding: utf_8 -*-

# codebook_test.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from codebook import *
from util import *
from util_absolute_path import *

class ChapterTestCase(unittest.TestCase):
    def setUp(self):
        self.chapter = Chapter()
        self._list = [69, 81, 80, 86, 71, 80, 86]    #'content' in CID
    
    def testAddUsedGlyphs(self):
        self.chapter.addUsedGlyphs(self._list)
    
    def testGetUsedGlyphs(self):
        self.chapter.addUsedGlyphs(self._list)
        l = self.chapter.getUsedGlyphs()
        l.index(69)
        l.index(81)
        l.index(80)
    
from codebook import Codebook
from pangotag import PangoTag

class CodebookTestCase(unittest.TestCase):
    def setUp(self):
        self.codebook = Codebook()
        self.content = u'This is content'
        
        tag_a = PangoTag()
        tag_a.setElementName(u'span')
        tag_a.addAttr(u'face', u'Arial')
        tag_b = PangoTag()
        tag_b.setElementName(u'i')
        tag_c = PangoTag()
        tag_c.setElementName(u'span')
        tag_c.addAttr(u'font_family', mincho_face)
        tag_c.addAttr(u'size', u'20')
        self.pangotag_list = [tag_a, tag_b, tag_c]
        
        self.cc = self.codebook.cipherize(self.pangotag_list, self.content)
    
    def testCipherize(self):
        cc_second = self.codebook.cipherize(self.pangotag_list, self.content)
        assert self.cc == cc_second, 'does not make same cipher'
        print 'must be cipherized: ' + cc_second
    
    def testCipherizeKanji(self):
        ck = self.codebook.cipherize(self.pangotag_list, u'これは、漢字カナ交じりの文章です。')
        print 'must be cipherized: ' + ck
    
    def testGetCipherDic(self):
        cl = list(self.content)
        csl = list(self.cc)
        dic = self.codebook.getCipherDic()
        for i in range(len(cl)):
            assert dic[cl[i]] == csl[i], '_cipherDic is not consistent'
            
    def testGetSubstituteFace(self):
        face = u'Courier'
        sf = self.codebook.getSubstituteFace(face)
        assert sf != face, 'not cipherized'
        sf_a = self.codebook.getSubstituteFace(face)
        assert sf_a == sf, 'cipherize is not consistent'
        
    def testAddUsedFaceGlyphs(self):
        self.codebook.addUsedFaceGlyphs(mincho_file, 0, [69, 81, 80, 86, 71, 80, 86])    #'content' in CID
        assert self.codebook._chapterDic[FaceId(mincho_file, 0)]._usedGlyphList[69] == True

    def testGetChapter(self):
        self.codebook.cipherize(self.pangotag_list, u'これは、漢字カナ交じりの文章です。')
        c = self.codebook.getChapter(FaceId(mincho_file, 0))
        assert c._usedGlyphList[1998] == True    # 1998 is 'こ'
        
if __name__ == "__main__":
    unittest.main()
