# fonthandlerimpl_test.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from fonthandlerimpl import FontHandlerImpl
from scenariohandler import ScenarioHandler
from codebook import Codebook, ProxyCodebook
from util import *
from util_absolute_path import *
import unittest
from fontTools import ttx
from fontTools.ttLib import TTFont
import os
from ttcheader import *

class FontHandlerImplTestCase(unittest.TestCase):
    def setUp(self):
        self._fhi = FontHandlerImpl()
        # make codebook for test
        self._cb = Codebook()
        o = file(u'test_out.xml', 'w')
        parser = ScenarioHandler.getParser(o, self._cb)
        f = file(u'test_vn.xml', 'r')
        parser.parse(f)
        f.close()
        o.close()
        self._chapter = self._cb.getChapter(FaceId(mincho_file, 0))
        self._chapter_hg = self._cb.getChapter(FaceId(seikai_file, 0))

    def testSAXParser(self):
        pcb = ProxyCodebook()
        pcb.setCodebook(self._cb)
        cd = self._cb.getChapterDic()
        sub_index = 0
        for faceid, chapter in cd.iteritems():
            filename = faceid.getName()
            tt = None
            ttc = None
            if filename[-3:] == 'ttc' or filename[-3:] == 'TTC':
                ttc = TTCFont(filename)
                tt = ttc.getTTFont(faceid.getIndex())
            else:
                tt = TTFont(filename.encode())
            if filename == mincho_file and faceid.getIndex() == 0 and os.path.exists('msmincho.ttx'):
                r = self._fhi.run('msmincho.ttx', 'temp_subset.ttx', pcb, chapter)
            else:
                print 'Dumping font file: %s' % filename
                tt.saveXML('temp.ttx')
                r = self._fhi.run('temp.ttx', 'temp_subset.ttx', pcb, chapter)
            assert r == 0
            tt = TTFont()
            tt.importXML('temp_subset.ttx')
            tt.save('temp_subset.ttf')
            new_filename = 'test_out_' + str(sub_index) + '.ttf'
            if os.path.exists(new_filename):
                os.remove(new_filename)
            os.rename('temp_subset.ttf', new_filename)
            if os.path.exists('temp.ttx'):
                os.remove('temp.ttx')
            os.remove('temp_subset.ttx')
            sub_index = sub_index + 1


if __name__ == "__main__":
    unittest.main()
#import hotshot
#prof = hotshot.Profile("fonthandlerimpl_test.prof")
#benchtime, none = prof.runcall(unittest.main)
#prof.close()
