# -*- coding: utf_8 -*-

# pangosweeper_test.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
from pangosweeper import PangoSweeper
from pangotag import PangoTag
from codebook import Codebook, ProxyCodebook
from util import *
from util_absolute_path import *

test_str = u'これは、test用contentです。'

class MockCodebook:
    def __init__(self):
        self._expected = []

    def addExpected(self, face, index, cid_list):
        self._expected.append((face, index, cid_list))

    def addUsedFaceGlyphs(self, face, index, cid_list):
        try:
            a = self._expected.index((face, index, cid_list))
        except ValueError:
            print 'ERROR: you must specify addExpected() arguments: ', face, index, cid_list

class ProxyCodebookTestCase(unittest.TestCase):
    def setUp(self):
        self._proxy_codebook = ProxyCodebook()

    def testSetCodebook(self):
        cb = Codebook()
        self._proxy_codebook.setCodebook(cb)
        assert self._proxy_codebook._codebook == cb

class PangoSweeperTestCase(unittest.TestCase):
    def setUp(self):
        self._pangosweeper = PangoSweeper()
    
    def testSetCodebook(self):
        cb = Codebook()
        self._pangosweeper.setCodebook(cb)

    def getTags(self):
        tp = PangoTag()
        tp.setElementName(u'p')
        ts = PangoTag()
        ts.setElementName(u'span')
        ts.addAttr(u'face', mincho_face)
        return [tp, ts]

    def testSweepMock(self):
        cb = MockCodebook()
        self._pangosweeper.setCodebook(cb)
        cb.addExpected(mincho_file, 0, [1998, 2055, 2026, 1940])
        cb.addExpected(mincho_file, 0, [86, 71, 85, 86])
        cb.addExpected(mincho_file, 0, [8618])
        cb.addExpected(mincho_file, 0, [69, 81, 80, 86, 71, 80, 86])
        cb.addExpected(mincho_file, 0, [2018, 2004, 1941])
        self._pangosweeper.sweep(self.getTags(), test_str)

if __name__ == "__main__":
    unittest.main()
