/*
pangosweeperimpl.cpp -- Font subsetize tool for SDL_Pango.
Copyright (C) 2004 NAKAMURA Ken'ichi

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <boost/python.hpp>
#define PANGO_ENABLE_BACKEND
#include <pango/pango.h>
#include <pango/pangoft2.h>
#include <fontconfig/fontconfig.h>
#include <SDL_Pango.h>

using namespace boost::python;
using std::string;

struct PangoSweeperImpl
{
    object m_codebook;
    SDLPango_Context *m_context;

    PangoSweeperImpl()
    {
	if (! SDLPango_WasInit()) {
	    SDLPango_Init();
	}

	m_context = SDLPango_CreateContext();
    }

    ~PangoSweeperImpl()
    {
	SDLPango_FreeContext(m_context);
    }

    void setCodebook(object codebook)
    {
	m_codebook = codebook;
    }

    void addUsedFaceGlyphs(string Filename, int Index, list CharacterIDList)
    {
	m_codebook.attr("addUsedFaceGlyphs")(Filename, Index, CharacterIDList);
    }

    void sweep(string ContentWithTag)
    {
	SDLPango_SetMarkup(m_context, ContentWithTag.c_str(), -1);
	PangoLayoutIter *iter = pango_layout_get_iter (SDLPango_GetPangoLayout(m_context));
	do {
	    PangoLayoutLine *line = pango_layout_iter_get_line (iter);
	    GSList *tmp_list = line->runs;
	    while (tmp_list) {
		PangoLayoutRun *run = (PangoLayoutRun *)tmp_list->data;
		tmp_list = tmp_list->next;
		PangoFont *font = run->item->analysis.font;

		PangoFcFont *fcfont = (PangoFcFont *)font;
		FcPattern *pattern = fcfont->font_pattern;
		FcChar8 *filename;
		int id;
		if (FcPatternGetString (pattern, FC_FILE, 0, &filename) != FcResultMatch)
		    goto bail0;
		if (FcPatternGetInteger (pattern, FC_INDEX, 0, &id) != FcResultMatch)
		    goto bail0;

		PangoGlyphInfo *gi = run->glyphs->glyphs;
		list l;
		for(int i = 0; i < run->glyphs->num_glyphs; i++, gi++) {
		    if (gi->glyph)
			l.append(gi->glyph);
		}
		addUsedFaceGlyphs((const char*) filename, id, l);
	    }
	} while (pango_layout_iter_next_line (iter));
bail0:
	pango_layout_iter_free (iter);
	return;
    }
};


BOOST_PYTHON_MODULE(pangosweeperimpl)
{
    class_<PangoSweeperImpl>("PangoSweeperImpl")
		.def("sweep", &PangoSweeperImpl::sweep)
		.def("setCodebook", &PangoSweeperImpl::setCodebook)
    ;
}
