# scenariohandler.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from xml import sax
from xml.sax.handler import ContentHandler
from myxmlgen import XMLGenerator
from pangotag import PangoTag
from pangosweeper import PangoSweeper

class ScenarioHandler(XMLGenerator):
    """Cipherize scenario and build codebook."""
    pangoUri = 'http://fonthypo.sourceforge.jp/2004/0913/pango'

    def __init__(self, f):
        self._pangotag_list = []
        self._PangoSweeper = PangoSweeper()
        self._codebook = 0
        XMLGenerator.__init__(self, f, 'UTF-8')

    def setCodebook(self, codebook):
        self._codebook = codebook
        self._PangoSweeper.setCodebook(codebook)
    
    def getParser(cls, f, codebook):
        """Utitily function to get parser."""
        handler = ScenarioHandler(f)
        handler.setCodebook(codebook)
        parser = sax.make_parser()
        parser.setContentHandler(handler)
        parser.setFeature(sax.handler.feature_namespaces, True)
        return parser
    getParser = classmethod(getParser)
    
    def startElementNS(self, name, qname, attrs):
        uri, element = name
        if uri == ScenarioHandler.pangoUri:
            tag = PangoTag()
            tag.setElementName(element)
            for a, c in attrs.items():
                tag.addAttr(a[1], c)
            self._pangotag_list.append(tag)
            if element == u'span':
                # create new attrs
                new_attrs = {}
                for a, c in attrs.items():
                    if a[1] == u'face' or a[1] == u'font_family':
                        new_attrs[a] = self._codebook.getSubstituteFace(c)
                    else:
                        new_attrs[a] = c
                XMLGenerator.startElementNS(self, name, qname, new_attrs)
                return

        XMLGenerator.startElementNS(self, name, qname, attrs)
            
    def endElementNS(self, name, qname):
        XMLGenerator.endElementNS(self, name, qname)
        uri, element = name
        if uri == ScenarioHandler.pangoUri:
            self._pangotag_list.pop()

    def characters(self, content):
        if len(self._pangotag_list) > 0:
            self._PangoSweeper.sweep(self._pangotag_list, content)
            self._write(self._codebook.cipherize(self._pangotag_list, content))
        else:
            XMLGenerator.characters(self, content)
