# scenariohandler_test.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import unittest
import string
import os
from scenariohandler import *
from codebook import Codebook

pangoUri = 'http://fonthypo.sourceforge.jp/2004/0913/pango'
otherUri = 'http://fonthypo.sourceforge.jp/otherUri'


class ScenarioHandlerTestCase(unittest.TestCase):
    def setUp(self):
        self._o = os.tmpfile()
        class ProxyPangoSweeper(PangoSweeper):
            def __init__(self):
                self._is_sweep_called = False
                PangoSweeper.__init__(self)
            def sweep(self, pangotag_list, content):
                self._is_sweep_called = True
                PangoSweeper.sweep(self, pangotag_list, content)

    def tearDown(self):
        self._o.close()

    def testGetParser(self):
        parser = ScenarioHandler.getParser(self._o, Codebook())

    def testParse(self):
        parser = ScenarioHandler.getParser(self._o, Codebook())
        f = file(u'test_vn.xml', 'r')
        parser.parse(f)
        f.close()

    def testStartElementNSByOther(self):
        name = (otherUri, u'element')
        qname = u'ot:element'
        attr = {}
        handler = ScenarioHandler(self._o)
        handler.startElementNS(name, qname, attr)
        assert len(handler._pangotag_list) == 0
    
    def startElementNS_p(self, handler):
        name = (pangoUri, u'p')
        qname = u'pa:p'
        attr = {}
        handler.startElementNS(name, qname, attr)

    def endElementNS_p(self, handler):
        name = (pangoUri, u'p')
        qname = u'pa:p'
        handler.endElementNS(name, qname)

    def startElementNS_span(self, handler):
        name_s = (pangoUri, u'span')
        qname_s = u'pa:span'
        attr_s = {(None, u'face'):u'MS Mincho', (None, u'foreground'):u'red'}
        handler.startElementNS(name_s, qname_s, attr_s)

    def endElementNS_span(self, handler):
        name_s = (pangoUri, u'span')
        qname_s = u'pa:span'
        handler.endElementNS(name_s, qname_s)

    def testStartElementNSByPango(self):
        handler = ScenarioHandler(self._o)
        self.startElementNS_p(handler)
        assert len(handler._pangotag_list) == 1
        assert handler._pangotag_list[0].getElementName() == u'markup'
    
    def testEndElementNS(self):
        handler = ScenarioHandler(self._o)
        self.startElementNS_p(handler)
        self.endElementNS_p(handler)
        assert len(handler._pangotag_list) == 0
    
    def testNestedElement(self):
        handler = ScenarioHandler(self._o)
        handler.setCodebook(Codebook())
        self.startElementNS_p(handler)
        self.startElementNS_span(handler)
        self.endElementNS_span(handler)
        self.endElementNS_p(handler)
        assert len(handler._pangotag_list) == 0
    
    def testCharacters(self):
        class MockPangoSweeper:
            def __init__(self):
                self._is_sweep_called = False
            def sweep(self, pangotag_list, content):
                self._is_sweep_called = True
        handler = ScenarioHandler(self._o)
        handler.setCodebook(Codebook())
        mockps = MockPangoSweeper()
        handler._PangoSweeper = mockps
        handler.characters(u'This is non-pango tag content')
        assert mockps._is_sweep_called == False
        self.startElementNS_p(handler)
        handler.characters(u'This is pango tag content')
        c = handler.characters
        assert mockps._is_sweep_called == True

    def testSetCodebook(self):
        handler = ScenarioHandler(self._o)
        cb = Codebook()
        handler.setCodebook(cb)

if __name__ == "__main__":
    unittest.main()
