# subsetize.py -- Font subsetize tool for SDL_Pango.
# Copyright (C) 2004 NAKAMURA Ken'ichi
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from fonthandler import FontHandler
from scenariohandler import ScenarioHandler
from codebook import Codebook, ProxyCodebook
from util import *
from fontTools.ttLib import TTFont
import os
import sys
from ttcheader import *

def subsetize(outputdir, inputfiles):
    """Parse inputfiles and put results to outputdir.
    
    inputfiles are list of filenames. The files must be XML and
    contain Pango namespace.
    """
    fh = FontHandler()
    cb = Codebook()
    if not os.path.isdir(outputdir):
        os.makedirs(outputdir)

    for i in inputfiles:
        filepath = os.path.join(outputdir, i)
        o = file(filepath, 'w')
        parser = ScenarioHandler.getParser(o, cb)
        f = file(i, 'r')
        parser.parse(f)
        f.close()
        o.close()

    pcb = ProxyCodebook()
    pcb.setCodebook(cb)
    cd = cb.getChapterDic()
    sub_index = 0
    for faceid, chapter in cd.iteritems():
        filename = faceid.getName()
        tt = None
        ttc = None
        if filename[-3:] == 'ttc' or filename[-3:] == 'TTC':
            ttc = TTCFont(filename)
            tt = ttc.getTTFont(faceid.getIndex())
        else:
            tt = TTFont(filename.encode())
        print 'Dumping font file: %s' % filename.encode()
        tt.saveXML('temp.ttx')
        r = fh.run('temp.ttx', 'temp_subset.ttx', pcb, chapter)
        if r == 1:
            print 'This font is considered unembeddable. Abort.'
            sys.exit(1)
        elif r != 0:
            print 'Internal error. Abort.'
            sys.exit(1)
        tt = TTFont()
        tt.importXML('temp_subset.ttx')
        tt.save('temp_subset.ttf')
        sfile = os.path.join(outputdir, 'subset_' + str(sub_index) + '.ttf')
        if os.path.exists(sfile):
            os.remove(sfile)
        os.rename('temp_subset.ttf', sfile)
        os.remove('temp.ttx')
        os.remove('temp_subset.ttx')
        sub_index = sub_index + 1
