import struct
import types
from fontTools.ttLib import TTFont

class TTCHeaderReader:
    """Handle TTC header. It enables TTFont to handle TTC."""
    def __init__(self, file):
        if isinstance(file, types.StringTypes):
                file = open(file, "rb")
        else:
            pass # assume "file" is a readable file object
        self._file = file
        data = self._file.read(12)
        if len(data) <> 12:
            from fontTools import ttLib
            raise ttLib.TTLibError, "Not a TrueType or OpenType font (not enough data)"
        TTCTag, Version, numFonts = struct.unpack('>4sll', data)
        if TTCTag != 'ttcf':
            from fontTools import ttLib
            raise ttLib.TTLibError, "Not a TrueType Collection (bad magic number)"
        self._sfnt_offset = []
        for i in range(numFonts):
            data = self._file.read(4)
            offset, = struct.unpack('>l', data)
            self._sfnt_offset.append(offset)
    
    def getSFNT(self, index):
        self._file.seek(self._sfnt_offset[index])
        return self._file

class TTCFont:
    """Wrapper to TTCHeaderReader and TTFont."""
    def __init__(self, file=None):
        self._ttc = TTCHeaderReader(file)
    
    def getTTFont(self, sfnt_index=0, res_name_or_index=None,
            sfntVersion="\000\001\000\000", checkChecksums=0,
            verbose=0, recalcBBoxes=1):
        return TTFont(self._ttc.getSFNT(sfnt_index), res_name_or_index, 
            sfntVersion, checkChecksums, verbose, recalcBBoxes)
